﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using ANT_Managed_Library;

namespace ClockDriftCalculator
{
    /////////////////////////////////////////////////////////////////////////////////////
    /// Interaction logic for Window1.xaml.
    /// Manages the behaviour of Window1 -- the main (and only) app window.
    /////////////////////////////////////////////////////////////////////////////////////

    public partial class Window1 : Window
    {
        public static ANT_Device ANTDevice;
        public ANT_Channel ANTChannel;
        public Queue<ANT_Response> responseBuf = new Queue<ANT_Response>();
        // Timestamp byte
        public ushort TS;
        // Flag used when device connects to BAUD 50000(AP1) and exception is thrown.
        public bool flagANTDeviceError = false;
        // Flag used when connected device throws Capabilities Error.
        public bool flagCapabilitiesError = false;

        // Test variables
        public int NumberofMessages = 0;            
        public byte rxError = 0;
        public uint uiNetTimeStamp = 0;
        public ushort usPreviousTick = 0;
        public uint uiNetExpectedTimeStamp = 0;
        public uint uiRolloverEvents = 0;

        // Default channel, network and app parameters.
        public UInt16 deviceNumber = 33;
        public bool pairingOn = false;
        public Byte deviceType = 1;
        public Byte transmissionType = 1;
        public UInt16 msgPeriod = 8192;
        public Byte radioFreq = 66;
        public UInt16 testDuration = 10;

        public Window1()
        {
            InitializeComponent();
            ANT_Common.enableDebugLogs();
            this.Closed += new EventHandler(Window1_Closed);
            
            // Makes sure the window size is okay
            this.Height = 170;
            this.Width = 630;
            grid_Main.Height = 130;
            grid_Main.Width = 600;
        }


        void Window1_Closed(object sender, EventArgs e)
        {
            if (ANTDevice != null)
                ANTDevice.Dispose();
        }


        private void button_Clear_Click(object sender, RoutedEventArgs e)
        {
            PrintBox(textBox_StatusDisplay, "", false);
        }


        // "Connect" button handler.
        private void button_Connect_Click(object sender, RoutedEventArgs e)
        {
            // No device is connected at the moment.
            if ((string)button_Connect.Content == "Find USB Device")
            {
                try
                {
                    // Set Flag so that Catch knows what caused the exception.
                    flagANTDeviceError = true;          
                    ANTDevice = new ANT_Device((byte.Parse((string)combobox_USB.SelectionBoxItem)), 57600);

                    // Now that you are connected to either an AP2/AT3/CC2572, you must check to see if it has the the following capabilities.
                    // If its has both of the below capabilities, it supports TimeStamping.
                    flagCapabilitiesError = true;
                    if (!(ANTDevice.getDeviceCapabilities().ExtendedMessaging && ANTDevice.getDeviceCapabilities().ScanModeSupport))
                        throw new Exception("This Module does not support TimeStamping Capability");

                    // Create deviceResponse Handler
                    ANTDevice.deviceResponse += new ANT_Device.dDeviceResponseHandler(ANTDevice_deviceResponse);

                    rxError = 0;

                    // Update UI
                    string text = ANTDevice.getDeviceUSBInfo().printProductDescription() + " at USB " + combobox_USB.SelectionBoxItem.ToString() + " connected\n";
                    PrintBox(textBox_StatusDisplay, text, true);

                    ANT_DeviceCapabilities DeviceCapabilities = ANTDevice.getDeviceCapabilities();
                    PrintBox(textBox_StatusDisplay, DeviceCapabilities.ToString(), true);

                    ButtonText(button_Connect, "Release USB device");
                    combobox_USB.IsEnabled = false;
                    button_Connect.ToolTip = "Release the " + ANTDevice.getDeviceUSBInfo().printProductDescription();

                    // Make the Channel ID grid visible
                    grid_ChannelID.Visibility = Visibility.Visible;
                    
                    Resize(350, 385, 20);
                }

                catch (Exception ex)
                {
                    // If USB Connection exception
                    if (flagANTDeviceError)     
                    {
                        MessageBox.Show("Failed to connect! Potential causes: Device is either already in use, OR you are trying to connect to an AP1 module (Baud: 50000), OR no device is connected at the selected USB port. Try a different USB.\nNote: AP1 does not support time stamping");
                        flagANTDeviceError = false;
                    }
                    // USB Capabilities Exception
                    else if (flagANTDeviceError & flagCapabilitiesError)        
                    {
                        MessageBox.Show(ex.ToString() + " Try another USB");
                        flagANTDeviceError = false;
                        flagCapabilitiesError = false;
                    }
                    // Some other Exception
                    else                               
                        MessageBox.Show(ex.ToString());
                }
            }

            // Device is connected, disconnect it.
            else
            {
                string DeviceName = ANTDevice.getDeviceUSBInfo().printProductDescription();
                ANT_Device.shutdownDeviceInstance(ref ANTDevice);

                // Update UI
                ButtonText(button_Connect, "Find USB Device");
                combobox_USB.IsEnabled = true;
                string text = DeviceName + " at USB " + combobox_USB.SelectionBoxItem.ToString() + " released\n";
                text += "------------------------------------------------------------------\n";
                PrintBox(textBox_StatusDisplay, text, true);
                grid_testPropertiesResults.Visibility = Visibility.Hidden;
                grid_ChannelID.Visibility = Visibility.Hidden;
                grid_NetworkProperties.Visibility = Visibility.Hidden;
                Resize(170, 630, 20);
                // Reset the test variables
                ResetTest();

                // Reset Flags
                flagCapabilitiesError = false;
                flagANTDeviceError = false;
            }
        }


        // ANTDevice Response handler.
        void ANTDevice_deviceResponse(ANT_Response response)
        {
            responseBuf.Enqueue(response);
        }


        // "Open Rx Channel" button handler.
        private void button_openRxChannel_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                // The channel is closed at the moment.
                if ((string)button_openRxChannel.Content == "Begin Test")
                {
                    // Set TimeStamp
                    ANTDevice.setLibConfig(ANT_ReferenceLibrary.LibConfigFlags.MESG_OUT_INC_TIME_STAMP_0x20);

                    // Set Channel. Hardcoded to channel 0.
                    ANTChannel = ANTDevice.getChannel(0);
                    
                    ANTChannel.channelResponse += new dChannelResponseHandler(ANTChannel_channelResponse);
                    // Slave at networkNum 0
                    ANTChannel.assignChannel(ANT_ReferenceLibrary.ChannelType.BASE_Slave_Receive_0x00, 0);          

                    // Setting channel and device properties. This is useless if user manually set these earlier.
                    // Useful if user directly clicks on "open channel".
                    // The defaults will be used in that case.
                    ANTChannel.setChannelFreq(radioFreq);
                    ANTChannel.setChannelPeriod(msgPeriod);
                    ANTChannel.setChannelID(deviceNumber,pairingOn,deviceType,transmissionType);
                    // Search timeout of 10*2.5 = 25 seconds; chosen arbitrarily. 
                    ANTChannel.setChannelSearchTimeout(10);             

                    ResetTest();
                    ANTChannel.openChannel();

                    // Updating the UI
                    String text = "Channel opened with ID " + deviceNumber + "," + deviceType + "," + transmissionType;
                    if (pairingOn)
                        text += ", pairing";
                    text += "\nRF: 24" + radioFreq.ToString() + " MHz\nChannel Period: " + msgPeriod.ToString() + "/32768 [s]\n";
                    PrintBox(textBox_StatusDisplay, text, true);

                    ButtonText(button_openRxChannel, "Stop Test");
                    button_Connect.IsEnabled = false;
                    button_TestPropertiesResultsBack.IsEnabled = false;
                    button_TestDuration.IsEnabled = false;
                }

                // A channel is opened, close it.
                else                          
                {
                    ANTChannel.closeChannel();
                    textBox_StatusDisplay.ScrollToEnd();
                    PrintBox(textBox_StatusDisplay, "Channel closed\n", true);
                    button_openRxChannel.Content = "Begin Test";
                    button_Connect.IsEnabled = true;
                    button_TestPropertiesResultsBack.IsEnabled = true;
                    button_TestDuration.IsEnabled = true;
                    // Unassign handler
                    ANTChannel.channelResponse -= ANTChannel_channelResponse;
                }
            }

            catch (Exception ex)
            {
                // Most common error message; creating a separate message box.
                if (ex.Message == "Object reference not set to an instance of an object.")
                    MessageBox.Show("No ANT device is connected");
                
                // For all other errors.
                else
                    MessageBox.Show(ex.ToString());
            }
        }


        // "Set RF" button handler.
        private void button_setFreq_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                ANTChannel = ANTDevice.getChannel(0);
                funcSetFreq();
            }
            catch (Exception ex)
            {
                // Most common error message; creating a separate message box.
                if (ex.Message == "Object reference not set to an instance of an object.")
                    MessageBox.Show("No ANT device is connected");

                // For all other errors.
                else
                    MessageBox.Show(ex.Message, "Error Setting Frequency");
            }
        }


        // Helper function for setting the channel RF.
        private void funcSetFreq()
        {
            byte offset = getGUIFreq();
            if (ANTChannel.setChannelFreq(offset, 500))
            {
                textBox_StatusDisplay.ScrollToEnd();
                PrintBox(textBox_StatusDisplay, "Channel RF set: " + (2400 + offset).ToString() + "Mhz\n", true);
                radioFreq = offset;
            }
            else
            {
                MessageBox.Show("Set Channel Frequency Failed");
                throw new Exception("Set Channel Frequency Failed");
            }
        }


        // Helper function for reading the user-chosen RF from the GUI.
        private byte getGUIFreq()
        {
            try
            {
                return byte.Parse(textBox_setFreq.Text);
            }
            catch (Exception ex)
            {
                throw new Exception("Bad Frequency Parameter: " + ex.Message);
            }
        }


        // ANTChannel Response Handler
        void ANTChannel_channelResponse(ANT_Response response)
        {
            // Channel Response ID
            if (response.responseID == 0x40)     
            {
                // Channel Event
                if (response.messageContents[1] == 1)
                {
                    // Error Event
                    if (response.messageContents[2] == 2)
                    {
                        rxError++;
                        NumberofMessages++;
                        // Accouting for the lost message
                        uiNetExpectedTimeStamp += msgPeriod;

                        PrintBox(textBox_failnExpected, rxError.ToString(), false);
                    }

                    // Channel gone to search.
                    else if (response.messageContents[2] == 8)
                    {
                        // If channel gone to search mode due to too many RX Fails, close channel.
                        // Closing the channel Resets the NumberofMessages and RXFail Parameters and allows for a fresh session once the Channel is opened again.
                        // This mechanism allows my NumberofMessages to be in sync with the Broadcast/Ack and RxFail Event.

                        // Update UI
                        PrintBox(textBox_StatusDisplay, "Channel went to Search Mode \n", true);

                        // Close channel
                        ANTChannel.closeChannel();
                        ButtonText(button_openRxChannel, "Begin Test");
                        this.Dispatcher.BeginInvoke((Action)delegate { this.button_Connect.IsEnabled = true; }, System.Windows.Threading.DispatcherPriority.Background);

                        // Unassign handler
                        ANTChannel.channelResponse -= ANTChannel_channelResponse;
                    }

                    // Any other message code is not expected
                    else
                    {
                        ANTChannel.closeChannel();
                        // Unassign handler
                        ANTChannel.channelResponse -= ANTChannel_channelResponse;

                        ButtonText(button_openRxChannel, "Begin Test");
                        this.Dispatcher.BeginInvoke((Action)delegate { this.button_Connect.IsEnabled = true; }, System.Windows.Threading.DispatcherPriority.Background);

                        // Update UI
                        string output = "Unexpected message code: " + response.messageContents[2].ToString() + ". Try running the test again\n";
                        PrintBox(textBox_StatusDisplay, output, true);
                    }
                }

                else
                {
                    // Update UI
                    string output = "Response code " + response.messageContents[2].ToString() + " received in response to message ";
                    output += response.messageContents[1].ToString() + "from ANT device\n";
                    PrintBox(textBox_StatusDisplay, output, true);
                }
            }

            // Check that the message is either Broadcast or Ack
            else if ((response.responseID == 0x4E) || (response.responseID == 0x4F))
            {
                TS = (ushort)response.messageContents[10];
                TS |= (ushort)((response.messageContents[11]) << 8);

                NumberofMessages++;

                // If first message is received; used for setting zero (reference) point. 
                if (NumberofMessages == 1)
                {
                    PrintBox(textBox_OldTimeStamp, TS.ToString(), false);
                    usPreviousTick = TS;
                }
                else
                {
                    PrintBox(textBox_newTimeStamp, TS.ToString(), false);
                    this.Dispatcher.BeginInvoke((Action)delegate { this.Calculate(NumberofMessages, TS); }, System.Windows.Threading.DispatcherPriority.Background);  //calculate()
                }
            }

            // All other kinds of Rx related messages. Need to do nothing but increase the expected timestamp value
            else
            {
                // Update UI
                string output = "Unexpected message ID " + response.messageContents[1] + " received. Try running the test again \n";
                PrintBox(textBox_StatusDisplay, output, true);

                ANTChannel.closeChannel();
                // Unassign handler
                ANTChannel.channelResponse -= ANTChannel_channelResponse;

                ButtonText(button_openRxChannel, "Begin Test");
                this.Dispatcher.BeginInvoke((Action)delegate { this.button_Connect.IsEnabled = true; }, System.Windows.Threading.DispatcherPriority.Background);

            }
        }


        // Helper function used for performing the Clock Drift calculations.
        public void Calculate(int MessageCount, ushort usNewTick)
        {
            int iDifferencenewTick;
            double dppm;
            int idifferenceBetweenMessages;
            double uMessageFrequency = (double)32768 / (double)msgPeriod;

            // To account for calculate thread timing issue if channel is opened and closed repetitively very fast
            // This has no impact on the tests, just for preventing confusion through UI
            if (MessageCount > 0)
                idifferenceBetweenMessages = MessageCount - 1;
            else
                idifferenceBetweenMessages = 0;


            double ddifferenceBetweenMessagesTime = ((double)idifferenceBetweenMessages / uMessageFrequency);
            
            PrintBox(textBox_time, ddifferenceBetweenMessagesTime.ToString(), false);

            // The user chosen test duration is done. Perform the drift calculations.
            if (MessageCount > uMessageFrequency * testDuration)
            {
                // Calculating the clock drift in ppm.
                iDifferencenewTick = (int)(uiNetTimeStamp - uiNetExpectedTimeStamp);
                dppm = (((double)(1000000 / 32768) * iDifferencenewTick) / (ddifferenceBetweenMessagesTime));
                PrintBox(textBox_CalculatedDrift, dppm.ToString(), false);

                // Update UI
                PrintBox(textBox_StatusDisplay, "New Reading\n", true);
                // Close channel
                ANTChannel.closeChannel();
                ButtonText(button_openRxChannel, "Begin Test");
                button_Connect.IsEnabled = true;
                button_TestPropertiesResultsBack.IsEnabled = true;
                button_TestDuration.IsEnabled = true;
                // Unassign handler
                ANTChannel.channelResponse -= ANTChannel_channelResponse;
            }

            // Test duration is not finishes
            else
            {
                // If the TimeStamp value rolled over
                if (usNewTick < usPreviousTick)
                {
                    uiNetTimeStamp += (uint)(0xFFFF - (usPreviousTick - usNewTick - 1));
                    uiRolloverEvents++;
                    PrintBox(textBox_RolloverEvents, uiRolloverEvents.ToString(), false);
                }
                else
                    uiNetTimeStamp += (uint)(usNewTick - usPreviousTick);

                // Expected change in timestamp
                uiNetExpectedTimeStamp += msgPeriod;

                usPreviousTick = usNewTick;
            }
        }


        // Helper function that parses the given textbox as comma or dash single or double character hex code, ensuring it is 8-bytes long.
        static public byte[] parseTextBoxHex(TextBox textBox, int minLength, int maxLength)
        {
            // First we split the string by the delimiters.
            textBox.Text = textBox.Text.Replace(",", "-");
            string[] byteStrings = textBox.Text.Split(new char[] { '-' });

            // Handle parsing if not using delimiters.
            if (byteStrings.Length == 1 && byteStrings[0].Length > 2)
            {
                int numBytes = (int)Math.Ceiling(((double)byteStrings[0].Length) / 2);
                string[] hexParse = new string[numBytes];
                for (int j = 0; j < numBytes; ++j)
                {
                    hexParse[j] = byteStrings[0].Substring(j * 2, Math.Min(2, byteStrings[0].Length - j * 2));
                }
                byteStrings = hexParse;
            }

            // Make sure it isn't too long.
            if (byteStrings.Length > maxLength)
                throw new Exception("Can't parse, too many bytes");

            // Parse the given values as hex.
            byte[] toReturn = new byte[Math.Max(byteStrings.Length, minLength)];
            int i;
            for (i = 0; i < byteStrings.Length; ++i)
            {
                if (byteStrings[i].Equals(""))
                    throw new Exception("Empty byte in string, please ensure commas/dashes are placed appropriately");
                toReturn[i] = Convert.ToByte(byteStrings[i], 16);
            }

            // Fill with zeroes if it is not minLength bytes.
            while (i++ < minLength - 1)
                toReturn[i] = 0;

            // Save the parsed value to the box so user can see added zeroes, etc.
            textBox.Text = BitConverter.ToString(toReturn);
            return toReturn;
        }


        // "Set network key" button handler.
        private void button_SetNetworkKey_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                byte[] networkKey = parseTextBoxHex(textBox_NetworkKey, 8, 8);
                // Using 0 network number
                if (!ANTDevice.setNetworkKey(0, networkKey, 500))
                    throw new Exception("Failed to set network key");

                else
                {
                    PrintBox(textBox_StatusDisplay, "Network Key set: " + textBox_NetworkKey.Text + "\n", true);
                }
            }

            catch (Exception ex)
            {
                // Most common error message; creating a separate message box.
                if (ex.Message == "Object reference not set to an instance of an object.")
                    MessageBox.Show("No ANT device is connected");

                // For all other errors.
                else
                    MessageBox.Show(ex.Message, "Set Network Key Error");
            }
        }


        // "Set Channel ID" button handler.
        private void button_SetID_Click (object sender, RoutedEventArgs e)
        {
            try
            {
                // Parse from the channel ID UI elements
                deviceNumber = UInt16.Parse(textBox_deviceNum.Text);
                deviceType = Byte.Parse(textBox_deviceType.Text);
                transmissionType = Byte.Parse(textBox_transType.Text);
                pairingOn = checkBox_pairing.IsChecked.Value;
                ANTChannel = ANTDevice.getChannel(0);

                if (!ANTChannel.setChannelID(deviceNumber, pairingOn, deviceType, transmissionType, 500))
                    throw new Exception("Set Channel ID Failed");
                
                else
                {
                    String result = "Channel ID set: " + deviceNumber + "," + deviceType + "," + transmissionType;
                    if (pairingOn)
                        result += ", pairing";
                    PrintBox(textBox_StatusDisplay, result + "\n", true);

                    // Hide the Channel ID grid and make the Network Properties grid visible.
                    grid_ChannelID.Visibility = Visibility.Hidden;
                    grid_NetworkProperties.Visibility = Visibility.Visible;
                    Resize(340, 630, 20);
                }

            }
            catch (Exception ex)
            {
                // Most common error message; creating a separate message box.
                if (ex.Message == "Object reference not set to an instance of an object.")
                    MessageBox.Show("No ANT device is connected");

                // For all other errors.
                else
                    MessageBox.Show(ex.Message, "Set Channel ID Failure");
            }

        }
                
        
        // "Set channel period" button handler.
        private void button_setMsgPeriod_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                ANTChannel = ANTDevice.getChannel(0);
                funcSetMsgPeriod();
            }
            catch (Exception ex)
            {
                // Most common error message; creating a separate message box.
                if (ex.Message == "Object reference not set to an instance of an object.")
                    MessageBox.Show("No ANT device is connected");

                // For all other errors.
                else
                    MessageBox.Show(ex.Message, "Error Setting Message Period");
            }
        }

        
        // Helper function for setting the channel message period.
        private void funcSetMsgPeriod()
        {
            UInt16 period32768ths = getGUIMsgPeriod();
            if (ANTChannel.setChannelPeriod(period32768ths, 500))
            {
                PrintBox(textBox_StatusDisplay, "Message Period set: " + period32768ths + "/32768 sec \n", true);
                msgPeriod = period32768ths;
            }
            else
            {
                MessageBox.Show("Setting Message Period Failed");
                throw new Exception("Setting Message Period Failed");
            }
        }

        
        // Helper function for reading the user chosen message period from the GUI.
        private UInt16 getGUIMsgPeriod()
        {
            try
            {
                return UInt16.Parse(textBox_PeriodIn32768ths.Text);
            }
            catch (Exception ex)
            {
                throw new Exception("Bad Message Period Parameter: " + ex.Message);
            }
        }


        // Channel message frequency textbox handler. Also changes the message period.
        void textBox_PeriodInHz_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (textBox_PeriodInHz.IsKeyboardFocused)
            {
                try
                {
                    double periodHz = double.Parse(textBox_PeriodInHz.Text);
                    
                    // Because the actual value is set in a ushort of 32768ths units, we have these limits.
                    if (periodHz < 0.5 || periodHz > 32768) 
                        throw new Exception("Period out of range");
                    ushort period32768ths = (ushort)Math.Round(32768 / periodHz);
                    
                    // We only get a zero if it is on a rollover past 65535, so we just keep 65535.
                    if (period32768ths == 0)    
                        period32768ths = 65535;
                    PrintBox(textBox_PeriodIn32768ths, period32768ths.ToString(), false);
                }
                catch (Exception)
                {
                    PrintBox(textBox_PeriodIn32768ths, "error", false);
                }
            }
        }


        // Channel message period textbox handler. Also changes the message frequency.
        void textBox_PeriodIn32768ths_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (textBox_PeriodIn32768ths.IsKeyboardFocused || sender == this)
            {
                try
                {
                    ushort period32768ths = ushort.Parse(textBox_PeriodIn32768ths.Text);
                    double periodHz = Math.Round((double)32768 / period32768ths, 3);
                    String rounded = periodHz.ToString();

                    // Handle the rounding so the number always fits in the display.
                    if (rounded.Length > 5 && !rounded.Equals("Infinity"))
                    {
                        if (rounded[4] == '.')
                            rounded = rounded.Remove(4);
                        else
                            rounded = rounded.Remove(5);
                    }
                    PrintBox(textBox_PeriodInHz, rounded.ToString(), false);
                }
                catch (Exception)
                {
                    PrintBox(textBox_PeriodInHz, "error", false);
                }
            }
        }


        // RF textbox handler.
        private void textBox_setFreq_TextChanged(object sender, TextChangedEventArgs e)
        {
            try
            {
                byte offset = getGUIFreq();
                int freq = 2400 + offset;
                label_FreqToSet.Content = "= " + freq + " Mhz";
            }

            catch (Exception)
            {
                // Catches the false exception during app launch when label_FreqToSet has not been created yet.
                // No action needed in that case; safe to ignore exception.
            }
        }


        // "Set Duration" button handler
        private void button_TestDuration_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                testDuration = UInt16.Parse(textBox_testDuration.Text);
                PrintBox(textBox_StatusDisplay, "Test duration set: " + testDuration + " s.\n", true);
            }

            catch (Exception)
            {
                PrintBox(textBox_testDuration, "error", false);
            }
        }

       
        // "Back" button handler in the Network Properties grid
        private void button_BackNetworkProperties_Click(object sender, RoutedEventArgs e)
        {
            // Hide the Network Properties grid and make the Channel ID grid visible.
            grid_ChannelID.Visibility = Visibility.Visible;
            grid_NetworkProperties.Visibility = Visibility.Hidden;
            Resize(350, 385, 20);
        }


        // "Next" button handler in the Network Properties grid
        private void button_NextNetworkProperties_Click(object sender, RoutedEventArgs e)
        {
            // Hide the Network Properties grid and make the Test Results grid visible.
            grid_testPropertiesResults.Visibility = Visibility.Visible;
            grid_NetworkProperties.Visibility = Visibility.Hidden;
            Resize(430, 385, 20);
        }


        // "Back" button handler in the Test Properties and Result grid
        private void button_TestPropertiesResultsBack_Click(object sender, RoutedEventArgs e)
        {
            // Hide the Test Results and make the Network Properties grid grid visible.
            grid_testPropertiesResults.Visibility = Visibility.Hidden;
            grid_NetworkProperties.Visibility = Visibility.Visible;
            Resize(340, 630, 20);
        }


        // Helper delegate
        delegate void dResize(int NewHeight, int NewWidth, int step);
        
        // Helps in smoothly resizing the windows
        private void Resize(int NewHeight, int NewWidth, int step)
        {
            if (!Dispatcher.CheckAccess())
            {
                Dispatcher.BeginInvoke(new dResize(Resize), NewHeight, NewWidth, step);
                return;
            }

            grid_Main.Width = NewWidth - 10;
            grid_Main.Height = NewHeight - 10;

            while ((this.Height != NewHeight) || (this.Width != NewWidth))
            {
                if (this.Height > NewHeight)
                {
                    if (this.Height - NewHeight > step)
                    {
                        this.Height -= step;
                    }
                    else
                    {
                        this.Height = NewHeight;
                    }
                }
                else if (this.Height < NewHeight)
                {
                    if (NewHeight - this.Height > step)
                    {
                        this.Height += step;
                    }
                    else
                    {
                        this.Height = NewHeight;
                    }
                }

                if (this.Width > NewWidth)
                {
                    if (this.Width - NewWidth > step)
                    {
                        this.Width -= step;
                    }
                    else
                    {
                        this.Width = NewWidth;
                    }
                }
                else if (this.Width < NewWidth)
                {
                    if (NewWidth - this.Width > step)
                    {
                        this.Width += step;
                    }
                    else
                    {
                        this.Width = NewWidth;
                    }
                }
            }
        }


        // Helper function that resets the test variables
        private void ResetTest()
        {
            // Reset the test variables
            NumberofMessages = 0;
            rxError = 0;
            uiNetTimeStamp = 0;
            usPreviousTick = 0;
            uiNetExpectedTimeStamp = 0;
            TS = 0;
            uiRolloverEvents = 0;
            PrintBox(textBox_CalculatedDrift, "", false);
            PrintBox(textBox_failnExpected, "0", false);
            PrintBox(textBox_OldTimeStamp, "", false);
            PrintBox(textBox_newTimeStamp, "", false);
            PrintBox(textBox_time, "0", false);
            PrintBox(textBox_RolloverEvents, "0", false);
        }


        // Helper delegate
        delegate void dPrintBox(TextBox textbox, string newText, bool append);


        // Method to handle changing the text of a given textbox
        private void PrintBox(TextBox textbox, string newText, bool append)
        {
            if (!Dispatcher.CheckAccess())
            {
                Dispatcher.BeginInvoke(new dPrintBox(PrintBox), textbox, newText, append);
                return;
            }

            if (append)
                textbox.Text += newText;
            else
                textbox.Text = newText;
            textbox.ScrollToEnd();
        }


        // Helper delegate
        delegate void dButtonText(Button button, string newText);


        // Method to handle changing the text of a given button
        private void ButtonText(Button button, string newText)
        {
            if (!Dispatcher.CheckAccess())
            {
                Dispatcher.BeginInvoke(new dButtonText(ButtonText), button, newText);
                return;
            }

            button.Content = newText;
        }

    }
}
